/*****************************************************************************
 * Copyright (c) 2015 CEA LIST and others.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Patrick Tessier (CEA LIST) - Initial API and implementation
 *   
 *****************************************************************************/

package org.eclipse.papyrus.sysml14.service.types.advice;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.gmf.runtime.emf.type.core.ElementTypeRegistry;
import org.eclipse.gmf.runtime.emf.type.core.IElementType;
import org.eclipse.gmf.runtime.emf.type.core.ISpecializationType;
import org.eclipse.gmf.runtime.emf.type.core.edithelper.AbstractEditHelperAdvice;
import org.eclipse.gmf.runtime.emf.type.core.requests.CreateRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.IEditCommandRequest;
import org.eclipse.papyrus.sysml14.modelelements.Conform;
import org.eclipse.papyrus.sysml14.modelelements.View;
import org.eclipse.papyrus.sysml14.modelelements.Viewpoint;
import org.eclipse.papyrus.sysml14.service.types.util.SysMLServiceTypeUtil;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Generalization;
import org.eclipse.uml2.uml.util.UMLUtil;

/**
 * 7.3.2.1 Conform
 * the service edit must implement the following constraints
 * [1] The general classifier must be an element stereotyped by Viewpoint.
 * [2] The specific classifier must be an element that is stereotyped by View.
 * this class satisfies the requirement{@papyrus.req org.eclipse.papyrus.sysml14.service.types#Req_001}
 *
 *7.3.2.7 View
 *[1] A view must only conform to a single viewpoint.
 * this class satisfies the requirement{@papyrus.req org.eclipse.papyrus.sysml14.service.types#Req_003}
 *
 */
public class ConformCreationEditHelperAdvice extends AbstractEditHelperAdvice {
	/**
	 * @see org.eclipse.gmf.runtime.emf.type.core.edithelper.AbstractEditHelperAdvice#approveRequest(org.eclipse.gmf.runtime.emf.type.core.requests.IEditCommandRequest)
	 *
	 * @param request
	 * @return
	 */
	@Override
	public boolean approveRequest(IEditCommandRequest request) {
		//only CreateRelationshipRequest are interested for conform
		if(!(request instanceof CreateRelationshipRequest)){
			return super.approveRequest(request);	
		}
		CreateRelationshipRequest createRelationshipRequest=(CreateRelationshipRequest)request;
		//only elementType conform are interested for conform
		IElementType elementType=createRelationshipRequest.getElementType();

		if (elementType instanceof ISpecializationType){
			ArrayList<IElementType> elementTypesHierarchy= new ArrayList<IElementType>();
			elementTypesHierarchy.add(elementType);
			ISpecializationType iSpecializationType=(ISpecializationType)elementType;
			elementTypesHierarchy.addAll(Arrays.asList(iSpecializationType.getAllSuperTypes()));

			if(!(elementTypesHierarchy.contains(ElementTypeRegistry.getInstance().getType(SysMLServiceTypeUtil.ORG_ECLIPSE_PAPYRUS_SYSML14_CONFORM)))){
				return super.approveRequest(request);	
			}
			{//This is a conform Element
				//test source
				if (!(createRelationshipRequest.getSource() instanceof Classifier)){			
					return false;
				}
				else{
					// this is a view, but has it a conform link?
					Conform conforLink=null;
					Classifier viewBaseClass= (Classifier)createRelationshipRequest.getSource();
					// Find Conform link
					EList<Generalization> generalizations = viewBaseClass.getGeneralizations();
					if (generalizations != null) {
						Iterator<Generalization> itGen = generalizations.iterator();
						while (itGen.hasNext() && (conforLink == null)) {
							conforLink = UMLUtil.getStereotypeApplication(itGen.next(), Conform.class);
						}
					}
					//Test to verify if the view has already a conform link.
					if( conforLink!=null){
						return false;
					}
				}
				//the source is ok
				if( UMLUtil.getStereotypeApplication((Classifier)createRelationshipRequest.getSource(), View.class)==null){
					return false;
				}

				//test target
				if(createRelationshipRequest.getTarget()==null){
					return true;
				}
				if (!(createRelationshipRequest.getTarget() instanceof Classifier)){
					return false;
				}
				return UMLUtil.getStereotypeApplication((Classifier)createRelationshipRequest.getTarget(), Viewpoint.class)!=null;
			}
		}
		return super.approveRequest(request);	
	}

}
