/*****************************************************************************
 * Copyright (c) 2015 CEA LIST.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *
 *		CEA LIST - Initial API and implementation
 *
 *****************************************************************************/
package org.eclipse.papyrus.sysml14.service.types.advice;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gmf.runtime.common.core.command.CompositeCommand;
import org.eclipse.gmf.runtime.common.core.command.ICommand;
import org.eclipse.gmf.runtime.common.core.command.UnexecutableCommand;
import org.eclipse.gmf.runtime.emf.type.core.ElementTypeRegistry;
import org.eclipse.gmf.runtime.emf.type.core.ISpecializationType;
import org.eclipse.gmf.runtime.emf.type.core.edithelper.AbstractEditHelperAdvice;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyElementRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.ReorientRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.ReorientRequest;
import org.eclipse.gmf.runtime.notation.Edge;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.papyrus.infra.services.edit.service.ElementEditServiceUtils;
import org.eclipse.papyrus.infra.services.edit.service.IElementEditService;
import org.eclipse.papyrus.sysml14.blocks.NestedConnectorEnd;
import org.eclipse.papyrus.sysml14.service.types.command.SetNestedPathCommand;
import org.eclipse.papyrus.sysml14.service.types.util.ConnectorUtil;
import org.eclipse.papyrus.sysml14.service.types.util.SysMLServiceTypeUtil;
import org.eclipse.papyrus.sysml14.service.types.util.UMLConnectorUtils;
import org.eclipse.papyrus.uml.service.types.utils.RequestParameterUtils;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.ConnectorEnd;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.util.UMLUtil;

/**
 * 8.3.2.3 Block
 * the service edit must implement the following constraints
 * isEncapsulated: Boolean [0..1]
 * If true, then the block is treated as a black box; a part typed by this black box can only be connected via its ports or
 * directly to its outer boundary. If false, or if a value is not present, then connections can be established to elements of
 * its internal structure via deep-nested connector ends.
 * this class satisfies the requirement{@papyrus.req org.eclipse.papyrus.sysml14.service.types#Req_007}
 *
 */
public class ConnectorReorientEditHelperAdvice extends AbstractEditHelperAdvice {

	private UMLConnectorUtils utils = new UMLConnectorUtils();

	/**
	 *
	 * @see org.eclipse.gmf.runtime.emf.type.core.edithelper.AbstractEditHelperAdvice#getBeforeReorientRelationshipCommand(org.eclipse.gmf.runtime.emf.type.core.requests.ReorientRelationshipRequest)
	 *
	 * @param request
	 * @return
	 */
	@Override
	protected ICommand getBeforeReorientRelationshipCommand(final ReorientRelationshipRequest request) {
		final CompositeCommand compositeCommand = new CompositeCommand("Destroy Connector View Command");

		// the UML Connector Edit Helper Advice destroys connector views when roles changes
		// Here, we destroys connectors views, when the path changes
		final EObject editedElement = request.getRelationship();
		if (editedElement instanceof Connector) {

			if (applySysMLRules(((Connector) editedElement).getOwner())) {
				final Connector connector = (Connector) editedElement;
				// verify the path now
				int reorientDirection = request.getDirection();
				Edge reorientedEdgeView = RequestParameterUtils.getReconnectedEdge(request);
				View newEndView = RequestParameterUtils.getReconnectedEndView(request);
				View oppositeEndView = null;
				if (reorientedEdgeView != null) {
					oppositeEndView = (reorientDirection == ReorientRequest.REORIENT_SOURCE) ? reorientedEdgeView.getTarget() : reorientedEdgeView.getSource();
				}
				List<Property> newNestedPath = null;
				if (reorientedEdgeView != null && newEndView != null) {
					// we are working with a 'graphical' reconnect request
					newNestedPath = utils.getNestedPropertyPath(newEndView, oppositeEndView);
				} else {
					newNestedPath = (List<Property>) request.getParameter(UMLConnectorUtils.NESTED_CONNECTOR_END_PATH);
				}

				final Set<View> viewsToDestroy = utils.getViewsRepresentingConnector(connector);
				final Iterator<View> iter = viewsToDestroy.iterator();

				while (iter.hasNext()) {
					final Edge current = (Edge) iter.next();
					if (current != reorientedEdgeView) {
						View oldEndView = null;
						if (reorientDirection == ReorientRequest.REORIENT_SOURCE) {
							oldEndView = current.getSource();
						} else if (reorientDirection == ReorientRequest.REORIENT_TARGET) {
							oldEndView = current.getTarget();
						}
						if (oppositeEndView != null) {
							final List<Property> oldNestedPath = utils.getNestedPropertyPath(oldEndView, oppositeEndView);
							if (!newNestedPath.equals(oldNestedPath)) {
								final DestroyElementRequest destroyRequest = new DestroyElementRequest(request.getEditingDomain(), current, false);
								final IElementEditService commandProvider = ElementEditServiceUtils.getCommandProvider(current);
								compositeCommand.add(commandProvider.getEditCommand(destroyRequest));
							}
						}
					}
				}
				if (!compositeCommand.isEmpty()) {
					return compositeCommand;
				}
			}
		}
		return null;
	}

	/**
	 *
	 * @param connectorOwner
	 *            the owner of the connector
	 * @return
	 */
	// FIXME : Bad name and should not use/define a qualified name here
	protected boolean applySysMLRules(final Element connectorOwner) {
		return connectorOwner.getApplicableStereotype("SysML::Blocks::Block") != null;
	}

	@Override
	protected ICommand getAfterReorientRelationshipCommand(final ReorientRelationshipRequest request) {
		// we set the new value for the nested path
		ICommand defaultCommand = super.getAfterReorientRelationshipCommand(request);

		int reorientDirection = request.getDirection();


		// Restrict this advice action to the end of Connector creation gesture (before clicking on target)
		// in order to add SysML specific constraint
		Connector connector = (Connector) request.getRelationship();
		// get the direction

		// Restrict action to SysML Connector (meaning owned by Block)
		if (((ISpecializationType) ElementTypeRegistry.getInstance().getType(SysMLServiceTypeUtil.ORG_ECLIPSE_PAPYRUS_SYSML14_BLOCK)).getMatcher().matches(connector.eContainer())) {
			List<Property> newFullNestedPath = new ArrayList<Property>();
			Edge reorientedEdgeView = RequestParameterUtils.getReconnectedEdge(request);
			View newEndView = RequestParameterUtils.getReconnectedEndView(request);
			View oppositeEndView = null;

			// graphical case : verify encapsulation and get the new path
			if (reorientedEdgeView != null) {

				oppositeEndView = (reorientDirection == ReorientRequest.REORIENT_SOURCE) ? reorientedEdgeView.getTarget() : reorientedEdgeView.getSource();
				if (newEndView != null && oppositeEndView != null) {
					// If the source or target view is enclosed in a structure encapsulated view, forbid creation.
					if (ConnectorUtil.isCrossingEncapsulation(newEndView, oppositeEndView) || ConnectorUtil.isCrossingEncapsulation(oppositeEndView, newEndView)) {
						return UnexecutableCommand.INSTANCE;
					}
					newFullNestedPath = utils.getNestedPropertyPath(newEndView, oppositeEndView);
				}
			} else {
				// we are in the semantic case
				newFullNestedPath = ((List<Property>) request.getParameter(UMLConnectorUtils.NESTED_CONNECTOR_END_PATH));

				// we need to verify encapsulation
				if (ConnectorUtil.isCrossingEncapuslation(newFullNestedPath)) {
					return UnexecutableCommand.INSTANCE;
				}
			}

			Property newRole = (Property) request.getNewRelationshipEnd();
			final ConnectorEnd oppositeEnd;
			if (reorientDirection == ReorientRequest.REORIENT_SOURCE) {
				oppositeEnd = ((Connector) request.getRelationship()).getEnds().get(1);
			} else {
				oppositeEnd = ((Connector) request.getRelationship()).getEnds().get(0);
			}
			UMLUtil.getStereotypeApplication(oppositeEnd, NestedConnectorEnd.class);
			newFullNestedPath.add(newRole);


			int tmpNestedPathDirection = (reorientDirection == ReorientRequest.REORIENT_SOURCE) ? SetNestedPathCommand.NESTED_SOURCE : SetNestedPathCommand.NESTED_TARGET;
			defaultCommand = CompositeCommand.compose(defaultCommand, new SetNestedPathCommand("Set connector nested source path", request.getRelationship(), request, newFullNestedPath, tmpNestedPathDirection));
		}

		return defaultCommand;
	}

}
