/*****************************************************************************
 * Copyright (c) 2012 CEA LIST.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *
 *		CEA LIST - Initial API and implementation
 *
 *****************************************************************************/
package org.eclipse.papyrus.sysml14.service.types.command;

import java.util.List;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gmf.runtime.common.core.command.CommandResult;
import org.eclipse.gmf.runtime.emf.type.core.ElementTypeRegistry;
import org.eclipse.gmf.runtime.emf.type.core.ISpecializationType;
import org.eclipse.gmf.runtime.emf.type.core.commands.EditElementCommand;
import org.eclipse.gmf.runtime.emf.type.core.requests.CreateRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.IEditCommandRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.ReorientRelationshipRequest;
import org.eclipse.papyrus.sysml14.blocks.BlocksPackage;
import org.eclipse.papyrus.sysml14.blocks.BoundReference;
import org.eclipse.papyrus.sysml14.blocks.NestedConnectorEnd;
import org.eclipse.papyrus.sysml14.service.types.util.SysMLServiceTypeUtil;
import org.eclipse.papyrus.sysml14.service.types.util.UMLConnectorUtils;
import org.eclipse.uml2.uml.ConnectableElement;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.ConnectorEnd;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.util.UMLUtil;
import org.eclipse.uml2.uml.util.UMLUtil.StereotypeApplicationHelper;

/**
 * 8.3.2.3 Block
 * the service edit must implement the following constraints
 * isEncapsulated: Boolean [0..1]
 * If true, then the block is treated as a black box; a part typed by this black box can only be connected via its ports or
 * directly to its outer boundary. If false, or if a value is not present, then connections can be established to elements of
 * its internal structure via deep-nested connector ends.
 * this class satisfies the requirement{@papyrus.req org.eclipse.papyrus.sysml14.service.types#Req_007}
 * 
 * 8.3.2.4 Bound Reference
 * set the BoundEnd
 * boundEnd : ConnectorEnd [1]
 * Gives a connector end of a binding connector opposite to the end linked to the stereotyped property, or linked to
 * property that generalizes the stereotyped one through redefinition.
 * this class satisfies the requirement{@papyrus.req org.eclipse.papyrus.sysml14.service.types#Req_008}
 */
public class SetNestedPathCommand extends EditElementCommand {

	private UMLConnectorUtils utils = new UMLConnectorUtils();

	/**
	 * The ordered list of {@link Property} to follow in order to reach the nested {@link ConnectableElement}.
	 */
	private List<Property> nestedPath;

	/**
	 * Indicates that the source of the {@link Connector} is nested.
	 */
	public static final int NESTED_SOURCE = 1;

	/**
	 * Indicates that the target of the {@link Connector} is nested.
	 */
	public static final int NESTED_TARGET = 2;

	/**
	 * Indicates whether or not the source of the target of the {@link Connector} is nested.
	 * One of {@link #NESTED_SOURCE} or {@link #NESTED_TARGET}.
	 */
	private final int nestedEndDirection;

	/**
	 *
	 * Constructor.
	 *
	 * @param label
	 *            the command label.
	 * @param elementToEdit
	 *            the element to edit.
	 * @param request
	 *            the request this command contribute to ({@link CreateRelationshipRequest} or {@link ReorientRelationshipRequest})
	 * @param nestedPath
	 *            the property path to set (in {@link NestedConnectorEnd#getPropertyPath()} ).
	 * @param nestedEndDirection
	 *            the direction ({@link #NESTED_SOURCE} or {@link #NESTED_TARGET}) of the {@link ConnectorEnd} to modify.
	 */
	public SetNestedPathCommand(String label, EObject elementToEdit, IEditCommandRequest request, List<Property> nestedPath, int nestedEndDirection) {
		super(label, elementToEdit, request);
		this.nestedPath = nestedPath;
		this.nestedEndDirection = nestedEndDirection;
	}

	/**
	 * Test whether the command can be executed or not.
	 */
	@Override
	public boolean canExecute() {

		// Verify the kind of request
		if (!(getRequest() instanceof CreateRelationshipRequest) && !(getRequest() instanceof ReorientRelationshipRequest)) {
			return false;
		}

		// if (!(getElementToEdit() instanceof Connector)) {
		// return false;
		// }

		// Verify (basic) that the related Connector well-formed.
		// In case of Connector creation, the connector is most probably not created yet
		if ((getConnector() != null) && (getConnector().getEnds().size() != 2)) {
			return false; // binary connector expected.
		}

		return super.canExecute();// true;
	}

	@Override
	protected CommandResult doExecuteWithResult(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		if (!canExecute()) {
			throw new ExecutionException("Unexecutable command."); //$NON-NLS-1$
		}

		ConnectorEnd modifiedEnd = getModifiedConnectorEnd();
		NestedConnectorEnd nestedConnectorEnd = UMLUtil.getStereotypeApplication(modifiedEnd, NestedConnectorEnd.class);

		if (nestedPath.isEmpty()) {
			// Remove stereotype if applied
			if (nestedConnectorEnd != null) {
				EcoreUtil.delete(nestedConnectorEnd, true);
			}

		} else {
			// Add stereotype if not applied and set new path if different from existing
			if (nestedConnectorEnd == null) {
				// apply stereotype
				nestedConnectorEnd = (NestedConnectorEnd) StereotypeApplicationHelper.getInstance(null).applyStereotype(modifiedEnd, BlocksPackage.eINSTANCE.getNestedConnectorEnd());
				// look for the opposite conectorEnd
				ConnectorEnd opposite = getNonModifiedConnectorEnd();
				ConnectableElement connectableElement = opposite.getRole();
				if (((ISpecializationType) ElementTypeRegistry.getInstance().getType(SysMLServiceTypeUtil.ORG_ECLIPSE_PAPYRUS_SYSML14_BOUNDREFERENCE)).getMatcher().matches(connectableElement)) {
					BoundReference defaultBoundReference = UMLUtil.getStereotypeApplication(connectableElement, BoundReference.class);
					defaultBoundReference.setBoundEnd(modifiedEnd);
				}


				// Set path
				if (!nestedConnectorEnd.getPropertyPath().equals(nestedPath)) {
					nestedConnectorEnd.getPropertyPath().clear();
					nestedConnectorEnd.getPropertyPath().addAll(nestedPath);
				}
			}
		}

		return CommandResult.newOKCommandResult(getConnector());
	}

	/**
	 * Get the {@link ConnectorEnd} currently modified by the command.
	 *
	 * @return the modified {@link ConnectorEnd}.
	 */
	private ConnectorEnd getModifiedConnectorEnd() {
		return (nestedEndDirection == NESTED_SOURCE) ? utils.getSourceConnectorEnd(getConnector()) : utils.getTargetConnectorEnd(getConnector());
	}

	/**
	 * Get the {@link ConnectorEnd} currently modified by the command.
	 *
	 * @return the modified {@link ConnectorEnd}.
	 */
	private ConnectorEnd getNonModifiedConnectorEnd() {
		return (nestedEndDirection == NESTED_SOURCE) ? utils.getTargetConnectorEnd(getConnector()) : utils.getSourceConnectorEnd(getConnector());
	}

	/**
	 * Get the {@link Connector} currently under creation or re-orient that hold {@link ConnectorEnd} to update.
	 *
	 * @return the related {@link Connector}.
	 */
	private Connector getConnector() {
		if (getElementToEdit() != null) {
			return (Connector) getElementToEdit();
		}
		if (getRequest() instanceof CreateRelationshipRequest) {
			return (Connector) ((CreateRelationshipRequest) getRequest()).getNewElement();
		} else if (getRequest() instanceof ReorientRelationshipRequest) {
			return (Connector) ((ReorientRelationshipRequest) getRequest()).getRelationship();
		}
		return null;

	}
}
